/* ------------------------------------------------------------------------------------------------ */
/* FILE :		 Demo_4_MSP5438_IAR.c																*/
/*																									*/
/* CONTENS :																						*/
/*				Demo for the Olimex MSP-5438STK evaluation board									*/
/*				Operation of the joystick shown on the LCD Display & both serial ports				*/
/*				(RS232 & RS232 over USB)															*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:		1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*																									*/
/*																									*/
/* Copyright (c) 2012-2014, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.4 $																				*/
/*	$Date: 2013/07/10 15:08:56 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 4
	#error "Wrong OS_DEMO value" 
#endif

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include "Abassi.h"

/* ------------------------------------------------ */

#include <msp430.h>

#define BAUDRATE	  (115200L)						/* Serial port baud rate						*/
#define LCD_FREQ	 (1000000L)						/* LCD serial port bit rate						*/
#define SYS_FREQ	 (9000000L)						/* Processor clock frequency					*/
#define OSC_FREQ	(18000000L)						/* Oscillator clock frequency					*/

void CLOCK_XT2_Init(void);
void Delay(unsigned long time);
void LCD_Clear();
void LCD_Clear_Memory();
void LCD_Contrast(int contrast);
void LCD_Init();
void LCD_Send(int data, int cmd);
void LCD_Update(int mode, int num, int y);
void LCD_Write(char *a, int x, int y);
void SPI2_Init();
void TIMERinit(long Rate);
void UARTinit(void);
void UART2_Init(unsigned long baud);
void UART3_Init(unsigned long baud);

#define SEND_CMD 			0						/* LCD commands									*/
#define SEND_CHR 			1
#define LCD_START_LINE_ADDR	(66-2)

/* ------------------------------------------------ */

int Debounce[5];									/* Used to debounce the joystick				*/
MBX_t *MbxUpDown;									/* Mailbox for the UP/DOWN task					*/
SEM_t *SemCenter;									/* Semaphore for CENTER task					*/
TSK_t *TaskCenter;									/* Task handling the joystick center activated	*/
TSK_t *TaskLeftRight;								/* Task handling the joystick R/L activated		*/
TSK_t *TaskTick;									/* Task operated one every RTOS tick			*/
TSK_t *TaskUpDown;									/* Task handling the joystick UP/DOWN activated	*/

/* ------------------------------------------------ */

void FctCenter(void);								/* Functon attached to TaskCenter				*/
void FctLeftRight(void);							/* Functon attached to TaskLeftRight			*/
void FctTick(void);									/* Functon attached to TaskTick					*/
void FctUpDown(void);								/* Functon attached to TaskUpDOwn				*/
void JoyStickHandler(void);							/* Joystick interrupt handler					*/

/* ------------------------------------------------ */

static const char FontLookup [][5] =		/* 5X7 fonts for the LCD display				*/
{
    { 0x00, 0x00, 0x00, 0x00, 0x00 },   // sp
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0xc4, 0xc8, 0x10, 0x26, 0x46 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // \ .
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 },   // z
    { 0x08, 0x6C, 0x6A, 0x19, 0x08 },   // {
    { 0x30, 0x4E, 0x61, 0x4E, 0x30 }    // |
};

char LCDMemory[504];
int  g_LCDidx;

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

int main(void)
{

/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	WDTCTL = WDTPW + WDTHOLD;						/* Turn off the watchdog timer					*/
													/* Done earlier in IAR with low_lelevl_init()	*/
	CLOCK_XT2_Init();

/* ------------------------------------------------ */
/* 	RTOS set-up										*/

	OSstart();

/* ------------------------------------------------ */
/* UART set-up										*/

	UARTinit();

/* ------------------------------------------------ */
/* Application set-up								*/


	TaskCenter    = TSKcreate("Center", 1, TC_TSK_STACK, &FctCenter,    1);
	TaskUpDown    = TSKcreate("UpDown", 1, TC_TSK_STACK, &FctUpDown,    1);
	TaskLeftRight = TSKcreate("LftRgh", 1, TC_TSK_STACK, &FctLeftRight, 1);
	TaskTick      = TSKcreate("Tick  ", 1, TC_TSK_STACK, &FctTick,      1);

	MbxUpDown     = MBXopen("UpDown", 3);			/* Create the mailbox used by FctUpDown()		*/

	SemCenter     = SEMopen("Center");				/* Create the semaphore used by FctCenter()		*/


/* ------------------------------------------------ */
/* LCD set-up										*/

	SPI2_Init();
	LCD_Init();

	LCD_Contrast(255);

	LCD_Write("Abassi Demo 4", 0, 0);
  	LCD_Update(1, 0, 0);
	LCD_Write(" Use Joystick", 0, 2);
	LCD_Update(1, 0, 2);

/* ------------------------------------------------ */
/* SysTick Set-up									*/

	P1DIR &= ~(0x7C);								/* Set joy stick pins as input					*/
	P1IES |=   0x7C;								/* Interrupt on high to low transition			*/
	P1IE  |=   0x7C;								/* Joy stick pin interrupt are enable			*/
	OSisrInstall(PORT1_VECTOR/2, &JoyStickHandler);	/* Install the joystick interrupt handler		*/

	TIMERinit(((SYS_FREQ/100000L)*((long)OS_TIMER_US))/10L);
	OSisrInstall(TIMER0_A0_VECTOR/2, &TIMtick);		/* Set-up & install the RTOS timer tick			*/

/* ------------------------------------------------ */

	puts("\n\n");									/* Welcome banner								*/
	puts("\n      Abassi Demo 4");
	puts("\n   Olimex MSP-5438STK\n");
	puts("\n    Use the joy stick\n\n");

/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	memset(&Debounce[0], sizeof(Debounce),0);

	OSeint(1);

	for (;;) {
		TSKselfSusp();								/* And Adam & Eve is done, it self suspends		*/
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{
	for(;;);										/* Idle is a do nothing task					*/
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Task functions																					*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctCenter(void)
{
	for (;;) {
		SEMwaitBin(SemCenter, -1);					/* Get blocked waiting for the joystick center	*/
													/* to be activated								*/
		printf("    Center\r");

		LCD_Write("    Center     ", 0, 4);
		LCD_Update(1, 0, 4);
	}
}


/* ------------------------------------------------------------------------------------------------ */

void FctLeftRight(void)
{
int events;

	for (;;) {
		EVTwait(0xFFFF, 0xFFFF, -1);				/* Wait for the event flags to be set by ISR	*/
		events = EVTget();
		if ((0x01 & events) != 0) {					/* Flag bit #0 is for right position			*/
			printf("    Right \r");

			LCD_Write("     Right     ", 0, 4);
			LCD_Update(1, 0, 4);
		}

		if ((0x10 & events) != 0) {					/* Flag bit #4 is for right position			*/
			printf("    Left  \r");

			LCD_Write("     Left      ", 0, 4);
			LCD_Update(1, 0, 4);
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */

void FctUpDown(void)
{
intptr_t Msg;

	for (;;) {
		MBXget(MbxUpDown, &Msg, -1);

		if ((int)Msg == 1) {						/* Message #1 is for up position				*/
			printf("    Up    \r");

			LCD_Write("      Up       ", 0, 4);
			LCD_Update(1, 0, 4);
		}
		if ((int)Msg == 2) {						/* Message #2 is for down position				*/
			printf("    Down  \r");

			LCD_Write("     Down      ", 0, 4);
			LCD_Update(1, 0, 4);
		}
	}
}

/* ------------------------------------------------------------------------------------------------ */

void FctTick(void)
{
int   ii;
const char Running[] = "|/-\\";

	ii = 0;
	for (;;) {
		EVTwait(0xFFFF, 0xFFFF, -1);				/* Wait for any event flag to be set by the		*/
													/* RTOS tierm callback							*/
		putchar(Running[ii++]);
		putchar('\r');
		ii &= 3;
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Joystick interrupt handler function																*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void JoyStickHandler(void)
{
int ii;

	ii = P1IV;										/* Get the vector indicating which pin			*/
	ii = (ii-6) >> 1;								/* Remap 6,8,10,12,14 -> 0,1,2,3,4				*/

	if (Debounce[ii] == 0) {						/* Is this pin currently being debounced		*/
		Debounce[ii] = OS_TICK_PER_SEC/50;			/* 20 ms debounce time							*/
		switch(ii) {								/* Trigger the corresponding task unless the	*/
		case 0:										/* pin is currrently in debounce				*/
			MBXput(MbxUpDown, 2, 0);				/* Down position								*/
			break;
		case 1:										/* Up position									*/
			MBXput(MbxUpDown, 1, 0);
			break;
		case 2:										/* Center position								*/
			SEMpost(SemCenter);
			break;
		case 3:										/* Right position								*/
			EVTset(TaskLeftRight, 0x01);
			break;
		case 4:										/* Left position								*/
			EVTset(TaskLeftRight, 0x10);
			break;
		default:
			break;
		}
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Timer call back function																			*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void TIMcallBack(void)
{
int ii;
static int CBcnt=0;


	if (++CBcnt > (OS_TICK_PER_SEC/10)) {			/* Trigger FctTick()once every 0.1 s			*/
		CBcnt=0;
		EVTset(TaskTick, 1);
	}


	for (ii=0 ; ii<5 ; ii++) {						/* Debouncing: Need to wait for the pin to go	*/
		if ((Debounce[ii] != 0)						/* back to a high level for long enough			*/
		&&  ((P1IN & (1<<(ii+2))) != 0)) {			/* Duration is set in the interrupt JoyStick()	*/
			Debounce[ii]--;
		}
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Initialization functions																			*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void CLOCK_XT2_Init()
{
int timeout;

	timeout = 5000;

	P11DIR  = 0x01
		    | 0x02
	        | 0x04;									/* set P11.0, P11.1 & P11.2 as output			*/
	P11SEL  = 0x01
		    | 0x02
		    | 0x04;									/* set P11.0/.1/.2 as ACLK/MCLK/SMCLK function	*/
	P5SEL   = P5SEL
		    | 0x04
		    | 0x08;									/* set P5.2 & P5.3 as XT2IN/XT2OUT				*/
	UCSCTL6 = 0x80									/* XT2 drive for 16 -> 24 MHz					*/
	        | 0x10;									/* XT2 soruce extyernally from the pin			*/
	UCSCTL3 = UCSCTL3 | 0x20;						/* set SELREF = 010 --> FLL clock src = REFOCLK	*/
	UCSCTL4	= 0x0005								/*	MCLK source == XT2CLK						*/
			| 0x0050								/* SMCLK source == XT2CLK						*/
			| 0x0500;								/*	ACLK source == XT2CLK						*/
	UCSCTL5 = 0x0001								/* MCLK : /2 									*/
//	        | 0x0004								/* SMCLK: /16									*/
	        | 0x0100;								/* ACLK : /2									*/
	do {
		if (!(UCSCTL7 & 0x08)) break;
		UCSCTL7 = UCSCTL7 & ~(0x0F);				/* Clear XT2,XT1,DCO fault flags				*/
		SFRIFG1 = SFRIFG1 & ~(0x02);				/* Clear fault flags (SFRIFG1.OFIFG = 0)		*/
		timeout--;
	} while ((SFRIFG1 & 0x02) && (timeout != 0));	/* Test oscillator fault flag					*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void TIMERinit(long Rate)
{
int PreSc;

	for (PreSc=0 ; PreSc<3 ; PreSc++) {				/* Find ther pre-scaling to use to have a count	*/
		if (Rate <= 65535L) {						/* held on 16 bits								*/
			break;
		}
		Rate = Rate >> 1;
	}

	if (Rate > 65535L) {
		Rate = 65535L;
	}
	else {
		Rate--;
	}

	TA0CTL	 = 0x0100								/* Timer A clock source is ACLK					*/
			 | (PreSc << 6)							/* Divider selection							*/
			 | 0x0010;								/* Mode up count								*/
	TA0CCR0	 = (int)Rate;
	TA0CCTL0 = CCIE;								/* Enable CCR0 interrupt of timer A0			*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void SPI2_Init(void)
{
unsigned int prescaler;

	P9SEL    |= 0x0E;								/* P9.1/P9.2/P9.3 set as alternate function		*/
	P9DIR    |= 0x0B;								/* P9.0/P9.1/P9.3 set as output					*/
	P9DIR    |= 0xC0;								/* LCD Reset, data/command bits set as output	*/
	
	UCB2CTL1 |= 0x01;								/* UCSWRST = 1 (reset state enable)				*/

	/* set format:																					*/
	/* set clock polarity and phase (UCCKPH = 0; UCCKPL = 1) --> bit7=0; bit6=1						*/
	/* MSB first; bit0 (UCSYNC) = 1; bit3 (master mode) = 1; --> bit5=1; bit4=0; bit3 =3; bit0=1;	*/
	/* set mode: 3pin SPI --> bit1=0; bit2=3;														*/
	/* UCB2CTL0 = 0b01101001 = 0x69																	*/

	UCB2CTL0  = 0x69;

	UCB2CTL1 |= 0x80;								/* Clock source - SMCLK							*/

  #if ((OSC_FREQ / LCD_FREQ) / 16) == 0
	prescaler = 1;
  #else
	prescaler = ((OSC_FREQ / LCD_FREQ) / 16);
  #endif
	UCB2BR0   = (prescaler &  0xff);
	UCB2BR1   = 0xFF & (prescaler >> 8);

	P9SEL    |= 0x0E;				  				/* P9.0/P9.1/P9.2/P9.3 set as alternate fct		*/
	P9DIR    |= 0x0B;								/* P9.0/P9.1/P9.3 set as output					*/

	UCB2CTL1 &=  ~(0x01);							/* UCSWRST = 0 (reset state disable)			*/

	return;

}

/* ------------------------------------------------------------------------------------------------ */
/* LCD functions																					*/
/* ------------------------------------------------------------------------------------------------ */

void LCD_Init()
{
	P9OUT = P9OUT & 0xBF;
	Delay(10000);
	P9OUT = P9OUT | 0x40;
	Delay(10000);


	LCD_Send(0x21, SEND_CMD);
	LCD_Send(0xC8, SEND_CMD);
	LCD_Send(0x04 | (!!(LCD_START_LINE_ADDR&(0x40))),   SEND_CMD);
	LCD_Send(0x40 | (LCD_START_LINE_ADDR & ((0x40)-1)), SEND_CMD);
	LCD_Send(0x12, SEND_CMD);
	LCD_Send(0x20, SEND_CMD);
	LCD_Send(0x08, SEND_CMD);
	LCD_Send(0x0C, SEND_CMD);

	LCD_Contrast(0xFF);
	LCD_Clear();
	LCD_Write(" ", 0, 0);

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Send(int data, int cmd)
{
	P9OUT = P9OUT & ~(0x01);

	if (cmd == SEND_CHR) {
		P9OUT |= 0x80;								/* Send char									*/
	}
	else {
		P9OUT &= 0x7F;								/* Send command									*/
	}

	while (!(UCB2IFG & 0x02));						/* Transmit the data over the SPI				*/
	UCB2TXBUF = data;

	P9OUT = P9OUT | 0x01;

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Clear_Memory()
{
	for (g_LCDidx=0; g_LCDidx<504; g_LCDidx++) {
		LCDMemory[g_LCDidx] = 0;
	}
	g_LCDidx = 0;

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Update(int mode, int num, int y)
{
char x;

	LCD_Send(0x80,	 SEND_CMD);
	LCD_Send(0x40|y, SEND_CMD);

	if (((mode == 1) && !(num & 1<<y))
	||  ((mode != 1) &&  (num & 1<<y))) {
		for(x=0; x<84; x++) {
			LCD_Send(LCDMemory[y*84+x], SEND_CHR);
		}
	}
	else {
		for(x=0; x<84; x++) {
			LCD_Send(~LCDMemory[y*84+x], SEND_CHR);
		}
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Clear()
{
int ii;

	LCD_Clear_Memory();
	for (ii=0 ; ii<6 ; ii++) {
		LCD_Update(1, 0x00, ii);
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Contrast(int contrast)
{
	LCD_Send(0x21, SEND_CMD);
	LCD_Send(0x80 | contrast, SEND_CMD);
	LCD_Send(0x20, SEND_CMD);

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void LCD_Write(char *sss, int x, int y)
{
char ii;

	y        = y + x/84;
	y        = y % 6;
	x        = x % 84;

	g_LCDidx = 84*y + x;

	LCD_Send(0x80 | x, SEND_CMD);
	LCD_Send(0x40 | y, SEND_CMD);

	while (*sss) {
		for (ii=0; ii<5; ii++) {					/* Charac are 5X7 pixels						*/
			if (g_LCDidx < 504) {
				LCDMemory[g_LCDidx++] = FontLookup[*sss-' '][ii];
			}
		}
		if (g_LCDidx < 504) {
			LCDMemory[g_LCDidx++] = 0x00;			/* 	space between characters					*/
		}
		sss++;
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */

void Delay(unsigned long time)
{

	while (time--);
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UARTinit(void)
{

	UART2_Init(BAUDRATE);

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void UART2_Init(unsigned long baud)
{
float n, x;

	P10SEL    = 0;
	P10DIR   |= (1<<7)								/* set P10.7 as output LED 1					*/
	         |  (1<<6);								/* set P10.6 as output LED 2					*/
	P9DIR    |= 1<<4;								/* set P9.4=1 (Transmitter) as output			*/
	P9DIR    &= ~(1<<5);							/* set P9.5=0 (Receiver) as input				*/
	P9SEL    |= (1<<4)								/* set P9.4=1 & P9.5=1 as TX/RX					*/
	         |  (1<<5);
	UCA2CTL1 |= 0x01;								/* UCSWRST = 1 (bit0 = 1)						*/
	UCA2CTL1 = (UCA2CTL1 | (1<<6) | 0x01)			/* select ACLK as BaudRate source clock			*/
	         & ~(1<<7);								/* (bit6 = 1; bit7 = 1) 						*/
	UCA2MCTL |= 0x01;								/* UCOS16 = 1									*/
	n        = SYS_FREQ / baud;						/* calculate UCBR								*/
	UCA2BR0  = (int)(n/16) % 256;
	UCA2BR1  = (int)(n/16) / 256;
	x        = (int)((n-(int)(n))*8+0.5);			/* calculate UCBRS								*/
	UCA2MCTL = UCA2MCTL | (int)(x)<<1;
	x        = (int)(((n/16 - (int)(n/16))*16) + 0.5);
	UCA2MCTL = UCA2MCTL | (int)(x)<<4;				/* calculate UCBRF								*/
													/* calculate registers for the BaudRate			*/
	UCA2CTL0 = 0x00;								/* Set Format Frame: 8 bit; no parity; 1 stop	*/
	UCA2CTL1 = UCA2CTL1 & ~(0x01);					/* UCSWRST = 0 (bit0 = 0)						*/

	return;
}

/* ------------------------------------------------------------------------------------------------ */

size_t __write(int dsc, unsigned char const *sss, size_t Size)
{
char ccc;
int	 ii;

	for (ii=0 ; ii<Size ; ii++) {					/* Write on both serial ports					*/
		ccc = sss[ii];
		if (ccc == '\n') {
			while((UCA2IFG & 0x02) == 0);
			UCA2TXBUF = '\r';
		}
		while (!(UCA2IFG & 0x02));
		UCA2TXBUF = ccc;
	}
	return(ii);
}

/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
int ccc;

	ccc = 0;

	if ((UCA2IFG & 0x01) != 0) {					/* Check both serial ports						*/
		ccc = UCA2RXBUF;
	}

	return(ccc);
}

/* EOF */
